<?php
namespace T33K\Restrictfe;

use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Core\Utility\ExtensionManagementUtility;

class Main
{

    public $extKey = 'restrictfe';

    /**
     * @var string
     */
    public $restrictfeEmConf = '';

    /**
     * @var \T33K\Restrictfe\Helper;
     */
    public $helper;

    /**
     * This is hook that is called at the very beginning of the FE rendering process
     *
     * @param $_params
     * @param $pObj
     *
     * @return void
     */
    public function redirectCheckForLoggedBeUser($_params, &$pObj)
    {
        // hook that allows you to enable restrictfe for example for database conditions
        if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['restrictfe/Class/Main.php']['restrictfe-PreProc'])) {
            $_params = array('pObj' => &$pObj);
            foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['restrictfe/Class/Main.php']['restrictfe-PreProc'] as $_funcRef) {
                GeneralUtility::callUserFunction($_funcRef, $_params, $this);
            }
        }

        if ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['enable']) {

            $this->helper = GeneralUtility::makeInstance('T33K\Restrictfe\Helper');
            $this->restrictfeEmConf = @unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf'][$this->extKey]);

            // 1 Early out on IP
            if (trim($this->restrictfeEmConf['ipAllowed'])) {
                if (GeneralUtility::cmpIP(GeneralUtility::getIndpEnv('REMOTE_ADDR'), $this->restrictfeEmConf['ipAllowed'])) {
                    return;
                }
            }

            // 2. Early out on header
            $headerAllowedArray = array();
            if (trim($this->restrictfeEmConf['headerAllowed'])) {
                $headerAllowedArray = GeneralUtility::trimExplode(',', trim($this->restrictfeEmConf['headerAllowed']));
            }
            if (in_array(trim($_SERVER['HTTP_TX_RESTRICTFE']), $headerAllowedArray)) {
                return;
            }

            // 3. Standard check for cookie presence
            $isTheRightCookieSet = FALSE;

            if ($_COOKIE['tx_restrictfe']) {
                /** @var \TYPO3\CMS\Core\Registry $registry */
                $registry = GeneralUtility::makeInstance('TYPO3\CMS\Core\Registry');
                $values = $registry->get('tx_restrictfe', intval($_COOKIE['tx_restrictfe']));
                if ($values['cookieSet']) {
                    $isTheRightCookieSet = TRUE;
                }
            }

            if (!$isTheRightCookieSet) {

                if (!$pObj->beUserLogin) {

                    $renderObj = GeneralUtility::makeInstance('TYPO3\CMS\Fluid\View\StandaloneView');
                    $renderObj->setTemplatePathAndFilename(PATH_site . $this->getRelativeTemplatePath() . 'show.html');
                    $renderObj->assign('beLoginLink', GeneralUtility::getIndpEnv('TYPO3_SITE_URL') . 'typo3/index.php?redirect_url=' . GeneralUtility::getIndpEnv('TYPO3_REQUEST_URL'));
                    $this->helper->setNoCacheHeaders();
                    $this->helper->setNoIndexHeaders();
                    $this->helper->setAccessForbiddenHeaders();

                    echo $renderObj->render();
                    exit;

                } else {

                    // We clear cookie of BE user in order to only authorize him in FE. That means that there is need for special BE account that have no privilages at all and is used only for that purpose.
                    $beUserRow = $this->helper->getBeUserRow();
                    if ($beUserRow['tx_restrictfe_clearbesession']) {
                        setcookie('be_typo_user', NULL, 1);
                    }

                    // create random cookie value and set it in registry to later check if this cookie has right to see frontend
                    $cookieValue = GeneralUtility::md5int(substr($GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey'], rand(1, 5), rand(5, 10)) . time());
                    setcookie('tx_restrictfe', $cookieValue, time() + 86400 * 100, '/', null, null, true);

                    /** @var \TYPO3\CMS\Core\Registry $registry */
                    $registry = GeneralUtility::makeInstance('TYPO3\CMS\Core\Registry');
                    $values = array('cookieSet' => 1);
                    $registry->set('tx_restrictfe', $cookieValue, $values);

                    header('Location:' . 'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI']);
                    exit;
                }
            }
        }
    }


    protected function getRelativeTemplatePath()
    {
        if (isset($this->restrictfeEmConf['templatesRelativePath']) && strlen(trim($this->restrictfeEmConf['templatesRelativePath']))) {
            $relativeTemplatePath = $this->restrictfeEmConf['templatesRelativePath'];
        } else {
            $relativeTemplatePath = ExtensionManagementUtility::siteRelPath($this->extKey) . 'Resources/Private/Templates/';
        }
        return $relativeTemplatePath;
    }

}